###############################################################################
# $Id: ATL.pm.src,v 1.7 1997/03/04 18:53:50 tom Exp $
# $Source: /RMM/RCS/vendors/odetics/pvr/scsi/ATL.pm.src,v $
###############################################################################
# Library Media Changer Output Formatter 
# ATL Vendor-specific Data and Subroutines
# This file is designed to be require'd by a front end script.
###############################################################################

###############################################################################
# Defaults/Global Constants/Global Variables
###############################################################################

# Common subroutines
%ATL_pages = (

	# Vendor Unique Page
	'0x20' => {
		# supported (changeable) parameters in page 0x20
		changeable 	=> [ qw(auto_clean drive_load_retry exb) ],
		dirty	   	=> undef,
		parse_fn	=> "parse_page_0x20",
		print_fn	=> "print_page_0x20",
		setattrs_fn	=> "setattrs_page_0x20",
		getattrs_fn	=> undef,
		write_fn	=> "write_page_0x20",
	},
	# Vendor Unique Page
	'0x00' => {
		# supported (changeable) parameters in page 0x00
		changeable 	=> [ qw(a_init nbl) ],
		dirty	   	=> undef,
		parse_fn	=> "parse_page_0x00",
		print_fn	=> "print_page_0x00",
		setattrs_fn	=> "setattrs_page_0x00",
		getattrs_fn	=> undef,
		write_fn	=> "write_page_0x00",
	},
);

###############################################################################
# Subroutines
###############################################################################

###############################################################################
# NAME:		parse_page_0x20
# DESCRIPTION:	Subroutine to parse the mode sense data for
#		Vendor Unique Page '0x20'. 
# INPUT ARGS:
#		1. Reference to initialized mode page structure
#		2. Data buffer
#		3. Page length
# RETURNS:
#		Reference to the mode page or 0 on error. 
###############################################################################
sub parse_page_0x20 {
	my ($ref, $rec, $mode_sense_param_len) = @_;
	my ($mode_sense_params) = "C" x $mode_sense_param_len;
	my ($data_byte2, $data_byte3);
	my ($auto_clean, $drive_load_retry); 

	# validate the reference
	ref($ref) eq "HASH" || dye(1, "expected page reference.\n");

	# extract the mode parameters for this page
        ($data_byte2, $data_byte3) = unpack($mode_sense_params, $rec);

	# Mask off the automatic drive cleaning bit
        $auto_clean = (0x02 & $data_byte2) == 0x02 ? 1 : 0;

	# Mask off the drive load retry bit
        $drive_load_retry = (0x01 & $data_byte2) == 0x01 ? 1 : 0;

	# Mask off the exabyte emulation bit
        $exb = (0x01 & $data_byte3) == 0x01 ? 1 : 0;

	# store the attributes for this page
	$ref->{'page_length'} = $mode_sense_param_len;
	$ref->{'auto_clean'} = $auto_clean;
	$ref->{'drive_load_retry'} = $drive_load_retry;
	$ref->{'exb'} = $exb;

	return $ref;
}

###############################################################################
# NAME:		print_page_0x20
# DESCRIPTION:	Subroutine to print the mode sense data for
#		Vendor Unique Page '0x20'. 
# INPUT ARGS:
#		1. Reference to mode page structure
#		2. Reference to the output file handle	
#		3. Output format specification
# RETURNS:
#		1 or die.
###############################################################################
sub print_page_0x20 {
	my ($ref, $ofh, $oformat) = @_;
	my ($auto_clean, $drive_load_retry); 

	# validate the reference
	ref($ref) eq "HASH" || dye(1, "expected page reference.\n");

	# print the attributes for this page.
	if ( $oformat eq "" ) {
		printf $ofh "\nVendor Unique Page 0x20\n";
		printf $ofh   "-----------------------\n";

		print "Page Code		[$ref->{'code'}]\n";
		print "PS-Page Saveable 	[$ref->{'saveable'}]\n";

		printf $ofh
		"Page Length		[0x%02.2x]\n", 
			$ref->{'page_length'};
		printf $ofh 
		"Auto Clean 		[%d]\n", 
			$ref->{'auto_clean'};
		printf $ofh "Drive Load Retry 	[%d]\n", 
			$ref->{'drive_load_retry'};
		printf $ofh "Exabyte Emulation	[%d]\n", 
			$ref->{'exb'};
	}
	elsif ( $oformat eq "colon" ) {
        	printf $ofh "0x20:%d:0x%02.2x:%d:%d:%d\n", 
			$ref->{'saveable'},
			$ref->{'page_length'},
			$ref->{'auto_clean'},
			$ref->{'drive_load_retry'},
			$ref->{'exb'};
	}

	return 1;
}

###############################################################################
# NAME:		parse_page_0x00
# DESCRIPTION:	Subroutine to parse the mode sense data for
#		Vendor Unique Page '0x00'. 
# INPUT ARGS:
#		1. Reference to initialized mode page structure
#		2. Data buffer
#		3. Page length
# RETURNS:
#		Reference to the mode page or 0 on error. 
###############################################################################
sub parse_page_0x00 {
	my ($ref, $rec, $mode_sense_param_len) = @_;
	my ($display_param) = $mode_sense_param_len - 2;
	my ($data_byte2); 
	my ($a_init, $u_init, $nbl, $nrdc, $mdc);
	my ($max_parity_retries, $display_message);

	# validate the reference
	ref($ref) eq "HASH" || dye(1, "expected page reference.\n");

	# extract the mode parameters for this page
        ($data_byte2, $max_parity_retries, $display_message) 
			= unpack( "3C a x $display_param", $rec);

	# Apply masks 
        $a_init = (0x80 & $data_byte2) == 0x80 ? 1 : 0;
        $u_init = (0x40 & $data_byte2) == 0x40 ? 1 : 0;
        $parity = (0x20 & $data_byte2) == 0x20 ? 1 : 0;
        $nbl = (0x10 & $data_byte2) == 0x10 ? 1 : 0;
        $nrdc = (0x02 & $data_byte2) == 0x02 ? 1 : 0;
        $mdc = (0x01 & $data_byte2) == 0x01 ? 1 : 0;

	# store the attributes for this page
	$ref->{'page_length'} = $mode_sense_param_len;
	$ref->{'a_init'} = $a_init;
	$ref->{'u_init'} = $u_init;
	$ref->{'parity'} = $parity;
	$ref->{'nbl'} = $nbl;
	$ref->{'nrdc'} = $nrdc;
	$ref->{'mdc'} = $mdc;
	$ref->{'max_parity_retries'} = $max_parity_retries;
	$ref->{'display_message'} = $display_message;

	return $ref;
}

###############################################################################
# NAME:		print_page_0x00
# DESCRIPTION:	Subroutine to print the mode sense data for
#		Vendor Unique Page '0x00'. 
# INPUT ARGS:
#		1. Reference to mode page structure
#		2. Reference to the output file handle	
#		3. Output format specification
# RETURNS:
#		1 or die.
###############################################################################
sub print_page_0x00 {
	my ($ref, $ofh, $oformat) = @_;
	my ($a_init, $nbl);

	# validate the reference
	ref($ref) eq "HASH" || dye(1, "expected page reference.\n");

	# print the attributes for this page.
	if ( $oformat eq "" ) {
		printf  $ofh "\nVendor Unique Page 0x00\n";
		printf  $ofh  "-----------------------\n";

		print 
		"Page Code				[$ref->{'code'}]\n";
		print 
		"PS-Page Saveable 			[$ref->{'saveable'}]\n";

		printf $ofh
		"Parameter Length			[0x%02.2x]\n", 
			$ref->{'page_length'};
		printf $ofh
		"AInit					[%d]\n", 
			$ref->{'a_init'};
		printf $ofh
		"UInit					[%d]\n", 
			$ref->{'u_init'};
		printf $ofh
		"Parity					[%d]\n", 
			$ref->{'parity'};
		printf $ofh
		"NBL-No Barcode Labels 			[%d]\n", 
			$ref->{'nbl'};
		printf $ofh 
		"NRDC - Not Ready Display Control	[%d]\n", 
			$ref->{'nrdc'};
		printf $ofh 
		"MDC - Message Display Control		[%d]\n", 
			$ref->{'mdc'};
		printf $ofh 
		"Maximum Parity Retries			[%d]\n", 
			$ref->{'max_parity_retries'};
		printf $ofh 
		"Display Message       			[%s]\n", 
		( $ref->{'display_message'} ? $ref->{'display_message'} : "" );
	}
	elsif ( $oformat eq "colon" ) {
        	printf $ofh "0x00:%d:0x%02.2x:%d:%d:%d:%d:%d:%d:%d:%s\n", 
			$ref->{'saveable'},
			$ref->{'page_length'},
			$ref->{'a_init'},
			$ref->{'u_init'},
			$ref->{'parity'},
			$ref->{'nbl'},
			$ref->{'nrdc'},
			$ref->{'mdc'},
			$ref->{'max_parity_retries'},
			$ref->{'display_message'};
	}

	return 1;
}

###############################################################################
# NAME:         write_page_0x20
# DESCRIPTION:  ATL-specific subroutine to write the mode select data 
#               for supported Vendor Unique Page '0x20'.
#
#		Vendor Unique Page 0x20h
#		+------------------------------------------+
#		| Bit/Byte | 7 | 6 | 5 | 4 | 3 | 2 | 1 | 0 |
#		+------------------------------------------+
#		|    0     |PS |(0)|   Page Code (20h)     |
#		+------------------------------------------+
#		|    1        Page Length (02h)            |
#		+------------------------------------------+
#		|    2        Reserved (00h)       |AC |DLR|
#		+------------------------------------------+
#		|    3        Reserved (00h)           |EXB|
#		+------------------------------------------+
#
# INPUT ARGS:
#               1. Reference to mode page structure
#               2. List of mode parameter attributes
#		   
# RETURNS:
#               1 or die.
###############################################################################
sub write_page_0x20 {
	my ($ref) = @_;
	my ($i);
	my ($mode_select_params) = "C" x 4;	# 4-byte block

	# validate the reference
	ref($ref) eq "HASH" || dye(1, "expected page reference.\n");

	#
	# build data block
	#
	
	# set the page code 
	$data_byte0 = 0x20; 

	# page length
	$data_byte1 = 0x02;
	$data_byte2 = ( (($ref->{'auto_clean'} & 0x01) << 1) | 
			($ref->{'drive_load_retry'} & 0x01) );

	$data_byte3 = ($ref->{'exb'} & 0x01);

	$verbose && do {
		print "\nSCSI Mode Select Data Block\n";
		print "---------------------------\n";
		printf("0x%02.2x 0x%02.2x 0x%02.2x 0x%02.2x\n", 
		 	$data_byte0, $data_byte1, $data_byte2, $data_byte3);
	};

	print pack $mode_select_params,  $data_byte0, $data_byte1,
		$data_byte2, $data_byte3;

	return 1;
}

###############################################################################
# NAME:         setattrs_page_0x20
# DESCRIPTION:  ATL-specific subroutine to set specified attributes
#               for supported Vendor Unique Page '0x20'.
#
#		Vendor Unique Page 0x20h
#		+------------------------------------------+
#		| Bit/Byte | 7 | 6 | 5 | 4 | 3 | 2 | 1 | 0 |
#		+------------------------------------------+
#		|    0     |PS |(0)|   Page Code (20h)     |
#		+------------------------------------------+
#		|    1        Page Length (02h)            |
#		+------------------------------------------+
#		|    2        Reserved (00h)       |AC |DLR|
#		+------------------------------------------+
#		|    3        Reserved (00h)           |EXB|
#		+------------------------------------------+
#
# INPUT ARGS:
#               1. Reference to mode page structure
#               2. List of mode parameter attributes
#		   
# RETURNS:
#               1 or die.
###############################################################################
sub setattrs_page_0x20 {
	my ($ref, @attrs) = @_;
	my ($i);
	my ($found) = 0;
	my ($mode_select_params) = "C" x 4;	# 4-byte block

	# supported (changeable) parameters in page 0x20
	my (@supported_params) = @{ $ATL_pages{'0x20'}{'changeable'} };

	# validate the reference
	ref($ref) eq "HASH" || dye(1, "expected page reference.\n");

	# validate the num of attrs
	dye(5, "must specify a minimum of one [Name=Value] pair")
		if ( !@attrs ); 

	foreach $i ( 0 .. $#attrs ) {
        	# split into key and value.
        	($key, $value) = split(/=/,$attrs[$i],2); 

		# validate the attribute
		dye(5, "invalid attribute page=0x20 [$key=$value] specified")
			if ( !is_member($key, \@supported_params) );

        	# print "key=$key, value=$value\n";

		# set the selected attribute's new value
		$ref->{$key} = $value;

		# found at least one valid attr
		$found++;

		# indicate page is dirty
		$ATL_pages{'0x20'}{'dirty'} = 1;
	}

	# print the attributes, with selections asserted, for this page.
	$verbose && do {
		printf  "\nSETATTRS Vendor Unique Page %s\n", $ref->{'code'};
		printf    "-------------------------------\n";
	
		print "Page Code		[$ref->{'code'}]\n";
		print "PS-Page Saveable 	[$ref->{'saveable'}]\n";
		printf  "Auto Clean 		[%d]\n", 
			$ref->{'auto_clean'};
		printf  "Drive Load Retry 	[%d]\n", 
			$ref->{'drive_load_retry'};
		printf  "Exabyte Emulation	[%d]\n", 
			$ref->{'exb'};
	};

	return 1;
}

###############################################################################
# NAME:         write_page_0x00
# DESCRIPTION:  ATL-specific subroutine to write the mode select data 
#               for supported Vendor Unique Page '0x00'.
#
#		Vendor Unique Page 0x00h
#		+------------------------------------------+
#		| Bit/Byte | 7 | 6 | 5 | 4 | 3 | 2 | 1 | 0 |
#		+------------------------------------------+
#		|    0     |PS |(0)|   Page Code (20h)     |
#		+------------------------------------------+
#		|    1        Page Length (3Eh)            |
#		+------------------------------------------+
#		|    2     |AIn|UIn|Par|NBL|Rsv|NRDC| MDC  |
#		+------------------------------------------+
#		|    3     Maximum Parity Retries (00h)	   |   
#		+------------------------------------------+
#		|    4 -    Display Messages		   |
#		|    63		(00h)			   |
#		+------------------------------------------+
#
# INPUT ARGS:
#               1. Reference to mode page structure
#               2. List of mode parameter attributes
#		   
# RETURNS:
#               1 or die.
###############################################################################
sub write_page_0x00 {
	my ($ref) = @_;
	my ($i);
	my ($mode_select_params) = "C" x 64;

	# validate the reference
	ref($ref) eq "HASH" || dye(1, "expected page reference.\n");

	#
	# build data block
	#
	
	# set the page code 
	$data_byte0 = 0x00; 

	# page length
	$data_byte1 = 0x3e;

	$data_byte2 = ( (($ref->{'a_init'} & 0x01) << 7) | 
			(($ref->{'nbl'} & 0x01) << 4) );

	# Zero-pad to the end of the block
	$db_null = 0;

	$verbose && do {
		print "\nSCSI Mode Select Data Block\n";
		print "---------------------------\n";
		printf("0x%02.2x 0x%02.2x 0x%02.2x 0x%02.2x\n", 
		 	$data_byte0, $data_byte1, $data_byte2, $data_byte3);
	};

	print pack $mode_select_params,  $data_byte0, $data_byte1,
		$data_byte2, ($db_null x 61);

	return 1;
}

###############################################################################
# NAME:         setattrs_page_0x00
# DESCRIPTION:  ATL-specific subroutine to set specified attributes
#               for supported Vendor Unique Page '0x00'.
#
# INPUT ARGS:
#               1. Reference to mode page structure
#               2. List of mode parameter attributes
#		   
# RETURNS:
#               1 or die.
###############################################################################
sub setattrs_page_0x00 {
	my ($ref, @attrs) = @_;
	my ($i);
	my ($found) = 0;
	my ($mode_select_params) = "C" x 4;	# 4-byte block

	# supported (changeable) parameters in page 0x00
	my (@supported_params) = @{ $ATL_pages{'0x00'}{'changeable'} };

	# validate the reference
	ref($ref) eq "HASH" || dye(1, "expected page reference.\n");

	# validate the num of attrs
	dye(5, "must specify a minimum of one [Name=Value] pair")
		if ( !@attrs ); 

	foreach $i ( 0 .. $#attrs ) {
        	# split into key and value.
        	($key, $value) = split(/=/,$attrs[$i],2); 

		# validate the attribute
		dye(5, "invalid attribute, page=0x00 [$key=$value] specified")
			if ( !is_member($key, \@supported_params) );

        	# print "key=$key, value=$value\n";

		# set the selected attribute's new value
		$ref->{$key} = $value;

		# found at least one valid attr
		$found++;

		# indicate page is dirty
		$ATL_pages{'0x00'}{'dirty'} = 1;
	}

	# print the attributes, with selections asserted, for this page.
	$verbose && do {
		printf  "\nSETATTRS Vendor Unique Page %s\n", $ref->{'code'};
		printf    "-------------------------------\n";
	
		print "Page Code		[$ref->{'code'}]\n";
		print "PS-Page Saveable 	[$ref->{'saveable'}]\n";
		printf  "AInit      		[%d]\n", 
			$ref->{'a_init'};
		printf  "NBL-No Barcode Labels	[%d]\n", 
			$ref->{'nbl'};
	};

	return 1;
}


